#include <stdlib.h>
#include <stdio.h>

#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qslider.h>
#include <qgroupbox.h>
#include <qtabwidget.h>
#include <qtextbrowser.h>
#include <qbuttongroup.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qtextstream.h>
#include <qstylefactory.h>

#include "tipografImpl.h"
#include "helpImpl.h"
#include "aboutImpl.h"


// ============================================================
// Main push button functions.
// ============================================================

/// Manages the operation of the Ok button.
void tipografImpl::ok()
{
  QString s = leInput->text();
  if(s.isNull()) {
    QMessageBox::warning(this,
			 tr("Print"),
			 tr("Please choose an input file first."),
			 QMessageBox::Ok, 0);  
    return;
  }
  // Check for running background processes.
  if(procView->isRunning() || procGen->isRunning()) {
    QMessageBox::warning(this,
			 tr("Print"),
			 tr("There are running background jobs.\n"
			    "Please quit the running preview program,\n"
			    "and start a new one afterwards."));
    return;
  }

  // Generate and check command line.
  QString cmd = "";
  if(commandLine() != 0) {
    return;
  }
  cmd = slCommandLine.join(" ");
  // Run a2ps.
  system(cmd.latin1());
}

/// Manages the operation of the Ok button.
void tipografImpl::preview()
{
  QString s = leInput->text();
  if(s.isNull()) {
    QMessageBox::warning(this,
			 tr("Preview"),
			 tr("Please choose an input file first."),
			 QMessageBox::Ok, 0);  
    return;
  }
  // Check for running background processes.
  if(procView->isRunning() || procGen->isRunning()) {
    QMessageBox::warning(this,
			 tr("Preview"),
			 tr("There are running background jobs.\n"
			    "Please quit the running preview program,\n"
			    "and start a new one afterwards."));
    return;
  }

  // Generate and check command line.
  QString cmd = "";
  if(commandLine() != 0) {
    return;
  }
  cmd = slCommandLine.join(" ") + " " +  sOutputToFileOpt + sTmpFileName;
  // Run a2ps.
  system(cmd.latin1());
  // Check if a2ps generated temporary file.
  FILE* f = fopen(sTmpFileName.latin1(), "r");
  if(f == NULL) {
    QMessageBox::critical(this,
			  tr("Preview"),
			  tr("a2ps did not generate any temporary file."));
    return;
  } else {
    fclose(f);
  }
  // Run PostScript Viewer.
  procView->clearArguments();
  procView->addArgument(lePathGv->text());
  procView->addArgument(sTmpFileName);
  if (!procView->start()) {
    QMessageBox::critical(this,
			  tr("Preview"),
			  tr("Unable to launch PostScript viewer."));
    return;
  }
}

/// Gets called when the background a2ps process exits.
void tipografImpl::readGen()
{
}

/// Gets called when the background PostScript viewer process exits.
void tipografImpl::readView()
{
  QString output = procView->readLineStdout();
  if(!procView->normalExit()) {
    QMessageBox::critical(this,
			  tr("Preview"),
			  tr("The PostScript viewer did not exit normally.\n"));
  }
  if(output != QString::null) {
    QMessageBox::critical(this,
			  tr("Preview"),
			  tr("The PostScript viewer issued the following\n"
			     "error message:\n\n")
			  + output);
  }
  QString cmd = "rm -f " + sTmpFileName;
  system(cmd.latin1());
}

void tipografImpl::help()
{
#ifdef HTMLDIR
  helpImpl h(this, "help", false);
  switch(cbTipoLanguage->currentItem()) {
  case 0:
    h.tbHelp->setSource(HTMLDIR "/en/index.html");
    break;
  case 1:
    h.tbHelp->setSource(HTMLDIR "/de/index.html");
    break;
  default:
    h.tbHelp->setSource(HTMLDIR "/en/index.html");
    break;
  }
  h.exec();
#else
#error HTMLDIR not defined. Help browser will not work.
#endif
}

void tipografImpl::exit()
{
  accept();
}


// ============================================================
// Input / Output tab.
// ============================================================
void tipografImpl::input()
{
  QString s = QFileDialog::getOpenFileName(QString::null,
					   QString::null,
					   this,
					   0,
					   tr("Open file"));
  if(s.isNull()) {
    QMessageBox::warning(this,
			 tr("Open file"),
			 tr("No valid file name chosen!"),
			 QMessageBox::Ok,
			 0);  
    return;
  }
  leInput->setText(s);
}

void tipografImpl::output()
{
  QString s = QFileDialog::getSaveFileName(QString::null,
					   QString::null,
					   this,
					   0,
					   tr("Save file"));
  if(s.isNull()) {
    QMessageBox::warning(this,
			 tr("Save file"),
			 tr("No valid file name chosen!"),
			 QMessageBox::Ok,
			 0);  
    return;
  }
  leOutput->setText(s);
}

void tipografImpl::outputTo()
{
  qWarning( "tipografImpl::outputTo() not yet implemented!" ); 
}

void tipografImpl::printer()
{
  tlPrinter->setText(slPrinterDesc[cbPrinter->currentItem()]);
}

/// Enables / disables the dialog tabs.
void tipografImpl::selfDefinedStyle()
{
  if(cbStdStyle->currentItem() == 1) {
    // Self defined
    for(int i=1; i<tabWidget->count()-2; i++) {
      tabWidget->setTabEnabled(tabWidget->page(i), true);
    }
    cbSimpleOptionsDisable->setEnabled(false);
    bgSimpleOptions->setEnabled(false);
    rbSimpleOptionPortrait->setEnabled(false);
    rbSimpleOptionLandscape->setEnabled(false);
    cbSimpleOptionLineNumbers->setEnabled(false);
    cbSimpleOptionColored->setEnabled(false);
    sbSimpleOptionFontSize->setEnabled(false);
    tlSimpleOptionFontSize->setEnabled(false);
  } else {
    // Std Style
    for(int i=1; i<tabWidget->count()-2; i++) {
      tabWidget->setTabEnabled(tabWidget->page(i), false);
    }
    bgSimpleOptions->setEnabled(true);
    cbSimpleOptionsDisable->setEnabled(true);
    simpleOptions();
  }
}

/// Get file type of input file.
void tipografImpl::type()
{
  QString s = leInput->text();
  if(s.isNull()) {
    QMessageBox::warning(this,
			 tr("File type"),
			 tr("Please choose an input file first."),
			 QMessageBox::Ok, 0);  
    return;
  }
  procType->clearArguments();
  procType->addArgument(lePathA2ps->text());
  procType->addArgument(sTypeOpt);
  procType->addArgument(s);
  
  if (!procType->start()) {
    QMessageBox::critical(this,
			  tr("File type"),
			  tr("Unable to start a2ps."));
    return;
  }
}

/// Checks exit status of a2ps -guess.
void tipografImpl::readType()
{
  QString output = procType->readLineStdout();
  if(output == QString::null) {
    QMessageBox::critical(0,
			  tr("File type"),
			  tr("Unable to read output from a2ps."));
    return;
  }
  leType->setText((output.section('(',-1)).section(')',0,0));
}

/// Enables / disables simple option controls.
void tipografImpl::simpleOptions()
{
  if(cbSimpleOptionsDisable->isChecked()) {
    rbSimpleOptionPortrait->setEnabled(false);
    rbSimpleOptionLandscape->setEnabled(false);
    cbSimpleOptionLineNumbers->setEnabled(false);
    cbSimpleOptionColored->setEnabled(false);
    sbSimpleOptionFontSize->setEnabled(false);
    tlSimpleOptionFontSize->setEnabled(false);
  } else {
    rbSimpleOptionPortrait->setEnabled(true);
    rbSimpleOptionLandscape->setEnabled(true);
    cbSimpleOptionLineNumbers->setEnabled(true);
    cbSimpleOptionColored->setEnabled(true);
    sbSimpleOptionFontSize->setEnabled(true);
    tlSimpleOptionFontSize->setEnabled(true);
  }
}



// ============================================================
// Title tab.
// ============================================================
/// Enables / disables header controls.
void tipografImpl::selfDefinedHeaders()
{
  if(cbHeadersEnable->isChecked()){
    leHeadLeft->setEnabled(true);
    leHeadCenter->setEnabled(true);
    leHeadRight->setEnabled(true);
    leWatermark->setEnabled(true);
    leFootLeft->setEnabled(true);
    leFootCenter->setEnabled(true);
    leFootRight->setEnabled(true);
    gbHeadLeft->setEnabled(true);
    gbHeadCenter->setEnabled(true);
    gbHeadRight->setEnabled(true);
    gbWatermark->setEnabled(true);
    gbFootLeft->setEnabled(true);
    gbFootCenter->setEnabled(true);
    gbFootRight->setEnabled(true);
  } else {
    leHeadLeft->setEnabled(false);
    leHeadCenter->setEnabled(false);
    leHeadRight->setEnabled(false);
    leWatermark->setEnabled(false);
    leFootLeft->setEnabled(false);
    leFootCenter->setEnabled(false);
    leFootRight->setEnabled(false);
    gbHeadLeft->setEnabled(false);
    gbHeadCenter->setEnabled(false);
    gbHeadRight->setEnabled(false);
    gbWatermark->setEnabled(false);
    gbFootLeft->setEnabled(false);
    gbFootCenter->setEnabled(false);
    gbFootRight->setEnabled(false);
  }
}



// ============================================================
// Layout tab.
// ============================================================
void tipografImpl::order()
{
  /* The iOrder int is used when generating the command line. */
  iOrder = slOrder->value();
  /* The string list "slOrderTxt" contains
     triples of strings representing the
     desired order.
  */
  tlOrder->setText(slOrderTxt[iOrder]);
}



// ============================================================
// Processing tab.
// ============================================================
/** Generates a triple of command line options.
 *    This is used to control the priority of the
 *    printing directives font size, rows per page
 *    and characters per line.
 */
QString tipografImpl::orderedOptions()
{
  static const QString errShouldNeverGetHere =
    tr("DEBUG:tipografImpl::orderedOptions(): control should never reach this point!");
  QString fs  = sFontSizeOpt    + sbFontSize->text();
  QString rpp = sRowsPerPageOpt + sbRowsPerPage->text();
  QString cpr = sCharsPerRowOpt + sbCharsPerRow->text();
  int o = slOrder->value();
  switch(o) {
  case 0:
    return(fs + " " + rpp + " " + cpr);
    break;
  case 1:
    return(fs + " " + cpr + " " + rpp);
    break;
  case 2:
    return(rpp + " " + fs + " " + cpr);
    break;
  case 3:
    return(rpp + " " + cpr + " " + fs);
    break;
  case 4:
    return(cpr + " " + fs + " " + rpp);
    break;
  case 5:
    return(cpr + " " + rpp + " " + fs);
    break;
  default:
    QMessageBox::critical(0,
			  tr("DEBUG"),
			  tr("DEBUG:tipografImpl::orderedOptions():\n"
			     "slOrder->value() returned unknown value.\n\n"
			     "Program will exit now."));
    reject();
  }
  return(errShouldNeverGetHere);
}

/// Reads out all options and generates an a2ps call.
int tipografImpl::commandLine()
{
  static const QString errIntro        =
    tr("The following errors occured:\n\n");
  static const QString errNoOutputFile =
    tr("You have chosen  \"Output to file\"\nbut failed to provide a file name.\n\n");
  static const QString errNoInputFile  =
    tr("You did not specify an input file.\n\n");
  QString errors = "";

  slCommandLine.clear();

  // Path to a2ps
  slCommandLine += lePathA2ps->text();

  // Output
  if(cbOutputTo->currentItem() == 1) {
    if(leOutput->text() != "") {
      slCommandLine += (sOutputToFileOpt + "\'" + leOutput->text() + "\'");
    } else {
      errors += errNoOutputFile;
    }
  } else {
    slCommandLine += slPrinterOpt[cbPrinter->currentItem()];
  }

  // StdStyles
  slCommandLine += slStdStyleOpt[cbStdStyle->currentItem()];

  if(cbStdStyle->currentItem() != 1) {
    if(cbSimpleOptionsDisable->isChecked() == false) {
    // Simple Options
    if(rbSimpleOptionPortrait->isChecked()) {
      slCommandLine += slOrientationOpt[1];
    } else {
      slCommandLine += slOrientationOpt[2];
    }
    if(cbSimpleOptionLineNumbers->isChecked()) {
      slCommandLine += sSimpleOptionLineNumbersOpt;
    }
    if(cbSimpleOptionColored->isChecked()) {
      slCommandLine += sSimpleOptionColoredOpt;
    }
    slCommandLine += (sFontSizeOpt + sbSimpleOptionFontSize->text());
    }
  } else {
    // Complex Options.
    slCommandLine += slOrientationOpt[cbOrientation->currentItem()];
    slCommandLine += slFormatOpt[cbFormat->currentItem()];
    slCommandLine += slDirectionOpt[cbDirection->currentItem()];
    slCommandLine += slDuplexOpt[cbDuplex->currentItem()];
    slCommandLine += (sRowsOpt + sbRows->text());
    slCommandLine += (sColumnsOpt + sbColumns->text());
    slCommandLine += (sMarginOpt + sbMargin->text());
    if(cbVirtualBorder->isChecked()) {
      slCommandLine += sVirtualBorderOpt;
    }
    if(lePSTitle->text() != "") {
      slCommandLine += (sPSTitleOpt + "\'" + lePSTitle->text() + "\'");
    }
    if(cbHeadersEnable->isChecked()) {
      if(leHeadLeft->text() != "") {
	slCommandLine += (sHeadLeftOpt + "\'" + leHeadLeft->text() + "\'");
      }
      if(leHeadCenter->text() != "") {
	slCommandLine += (sHeadCenterOpt + "\'" + leHeadCenter->text() + "\'");
      }
      if(leHeadRight->text() != "") {
	slCommandLine += (sHeadRightOpt + "\'" + leHeadRight->text() + "\'");
      }
      if(leWatermark->text() != "") {
	slCommandLine += (sWatermarkOpt + "\'" + leWatermark->text() + "\'");
      }
      if(leFootLeft->text() != "") {
	slCommandLine += (sFootLeftOpt + "\'" + leFootLeft->text() + "\'");
      }
      if(leFootCenter->text() != "") {
	slCommandLine += (sFootCenterOpt + "\'" + leFootCenter->text() + "\'");
      }
      if(leFootRight->text() != "") {
	slCommandLine += (sFootRightOpt + "\'" + leFootRight->text() + "\'");
      }
    }

    slCommandLine += orderedOptions();
    slCommandLine += slColorLevelOpt[cbColorLevel->currentItem()];
    slCommandLine += slCommentsOpt[cbComments->currentItem()];
    slCommandLine += slLanguageOpt[cbLanguage->currentItem()];
    slCommandLine += slUnprintablesOpt[cbUnprintables->currentItem()];
    slCommandLine += slPostScriptOpt[cbPostScript->currentItem()];
    slCommandLine += sTabSizeOpt + sbTabSize->text();
    slCommandLine += sLineNumbersOpt + sbLineNumbers->text();
    if(cbTOC->isChecked()) {
      slCommandLine += sTOCOpt;
    }
    if(cbTruncate->isChecked()) {
      slCommandLine += sTruncateOpt;
    }
  }
  // Processing Tab.
  if(cbEncoding->currentItem() != 0) {
    slCommandLine += slEncodingOpt[cbEncoding->currentItem()];
  }
  if(cbDelegate->isChecked()) {
    slCommandLine += sDelegateYesOpt;
  } else {
    slCommandLine += sDelegateNoOpt;
  }
  if(cbPrefeed->isChecked()) {
    slCommandLine += sPrefeedOpt;
  }
  slCommandLine += (sCopiesOpt + sbCopies->text());
  slCommandLine += leOptions->text();
  // Input
  if(leInput->text() != "") {
    slCommandLine += leInput->text();
  } else {
    errors += errNoInputFile;
  }
  tbCommandLine->setText(slCommandLine.join(" "));
  // Errors
  if(errors != "" && finishedStartUp == true) {
    QMessageBox::warning(this,
			 tr("Generate command line"),
			 errIntro + errors,
			 QMessageBox::Ok,
			 0);
    return(-1);
  }
  return(0);
}



// ============================================================
// Tipo tab.
// ============================================================
void tipografImpl::tipoLanguage()
{
  // Save ~/.tipografrc.
  saveTipo();
}

void tipografImpl::guiStyle()
{
  QApplication::setStyle
    (QStyleFactory::create
     (slGUIStyleTxt[cbGUIStyle->currentItem()]));
  // Save ~/.tipografrc.
  saveTipo();
}

void tipografImpl::pathA2ps()
{
  QString s = QFileDialog::getOpenFileName(QString::null,
					   QString::null,
					   this,
					   0,
					   tr("Path to a2ps"));
  if(s.isNull()) {
    QMessageBox::warning(this,
			 tr("Path to a2ps"),
			 tr("No valid file name chosen!"),
			 QMessageBox::Ok,
			 0);  
    return;
  }
  QFileInfo fi(s);
  if(!fi.isExecutable()) {
    QMessageBox::warning(this,
			 tr("Path to a2ps"),
			 tr("The file you chose is not executable."),
			 QMessageBox::Ok,
			 0);  
    return;
  }
  lePathA2ps->setText(fi.absFilePath());
  saveTipo();
}

void tipografImpl::pathGv()
{
  QString s = QFileDialog::getOpenFileName(QString::null,
					   QString::null,
					   this,
					   0,
					   tr("Path to PostScript viewer"));
  if(s.isNull()) {
    QMessageBox::warning(this,
			 tr("Path to PostScript viewer"),
			 tr("No valid file name chosen!"),
			 QMessageBox::Ok,
			 0);  
    return;
  }
  QFileInfo fi(s);
  if(!fi.isExecutable()) {
    QMessageBox::warning(this,
			 tr("Path to PostScript viewer"),
			 tr("The file you chose is not executable."),
			 QMessageBox::Ok,
			 0);  
    return;
  }
  lePathGv->setText(fi.absFilePath());
  saveTipo();
}

void tipografImpl::configFile()
{
  QString s = QFileDialog::getOpenFileName(QString::null,
					   QString::null,
					   this,
					   0,
					   tr("Path to config file"));
  if(s.isNull()) {
    QMessageBox::warning(this,
			 tr("Path to config file"),
			 tr("No valid file name chosen!"),
			 QMessageBox::Ok,
			 0);  
    return;
  }
  QFileInfo fi(s);
  if(!fi.isReadable()) {
    QMessageBox::warning(this,
			 tr("Path to config file"),
			 tr("The file you chose is not readable."),
			 QMessageBox::Ok,
			 0);  
    return;
  }
  leConfig->setText(fi.absFilePath());
}

void tipografImpl::about()
{
  aboutImpl a(this, "about", true);
  a.exec();
}
