#include <stdlib.h>
#include <stdio.h>

#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qslider.h>
#include <qgroupbox.h>
#include <qtabwidget.h>
#include <qtextbrowser.h>
#include <qbuttongroup.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qstylefactory.h>

#include <config.h>

#include "tipografImpl.h"
#include "init.h"

// ============================================================
// Config file functions.
// These relate to config files that hold printing
// control values like font size and so on. They save / load
// the values of all but the Tipo tab.
// See loadTipo / saveTipo for I/O to ~/.tipografrc.
// ============================================================

void tipografImpl::loadConfig()
{
  // Prepare file and error objects.
  QFile configFile(leConfig->text());
  QString parseErrIntro = tr("The following errors occured:\n\n");
  QString parseErr      = "";

  // Check config file.
  if(!configFile.exists()) {
    QMessageBox::warning(this,
			 tr("Load configuration"),
			 tr("The configuration file you are\n"
			    "trying to load does not exist."),
			 QMessageBox::Ok,
			 0);
    return;
  }
  
  if(!configFile.open(IO_ReadOnly)) {
    QMessageBox::warning(this,
			 tr("Load configuration"),
			 tr("Unable to open file for reading."),
			 QMessageBox::Ok,
			 0);
    return;
  }
  
  // Read in configuration.
  QTextStream s(&configFile);
  QString l;
  int equalPos = 0;
  int convInt = 0;
  bool convOk = false;

  while(!s.eof()) {
    l = s.readLine();
    if(l.contains("=")) {
      equalPos = l.find("=");

      // Input / output tab.
      if(l.contains("leInput")) {
	l = l.remove(0, equalPos+1);
	leInput->setText(l);
      }
      if(l.contains("leOutput")) {
	l = l.remove(0, equalPos+1);
	leOutput->setText(l);
      }
      if(l.contains("cbStdStyle")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbStdStyle->count() > convInt && convInt >= 0) {
	    cbStdStyle->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("Standard style index is out of range.\n");
	  }
	} else {
	  parseErr += tr("Standard style index is not a number.\n");
	}
      }
      if(l.contains("cbOutputTo")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbOutputTo->count() > convInt && convInt >= 0) {
	    cbOutputTo->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("Output to index is out of range.\n");
	  }
	} else {
	  parseErr += tr("Output to index is not a number.\n");
	}
      }
      if(l.contains("cbPrinter")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbPrinter->count() > convInt && convInt >= 0) {
	    cbPrinter->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("Printer index is out of range.\n");
	  }
	} else {
	  parseErr += tr("Printer index is not a number.\n");
	}
      }
      if(l.contains("cbSimpleOptionsDisable")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  switch(convInt) {
	  case 0:
	    cbSimpleOptionsDisable->setChecked(false);
	    break;
	  case 1:
	    cbSimpleOptionsDisable->setChecked(true);
	    break;
	  default:
	    parseErr += tr("Simple options disable value needs to be either one or zero.\n");
	  }
	} else {
	  parseErr += tr("Simple options disable value is not a number.\n");
	}
      }
      if(l.contains("rbSimpleOptionPortrait")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  switch(convInt) {
	  case 0:
	    rbSimpleOptionPortrait->setChecked(false);
	    break;
	  case 1:
	    rbSimpleOptionPortrait->setChecked(true);
	    break;
	  default:
	    parseErr += tr("Simple option portrait value needs to be either one or zero.\n");
	  }
	} else {
	  parseErr += tr("Simple option portrait value is not a number.\n");
	}
      }
      if(l.contains("rbSimpleOptionLandscape")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  switch(convInt) {
	  case 0:
	    rbSimpleOptionLandscape->setChecked(false);
	    break;
	  case 1:
	    rbSimpleOptionLandscape->setChecked(true);
	    break;
	  default:
	    parseErr += tr("Simple option landscape value needs to be either one or zero.\n");
	  }
	} else {
	  parseErr += tr("Simple option landscape value is not a number.\n");
	}
      }
      if(l.contains("cbSimpleOptionLineNumbers")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  switch(convInt) {
	  case 0:
	    cbSimpleOptionLineNumbers->setChecked(false);
	    break;
	  case 1:
	    cbSimpleOptionLineNumbers->setChecked(true);
	    break;
	  default:
	    parseErr += tr("Simple option line numbers value needs to be either one or zero.\n");
	  }
	} else {
	  parseErr += tr("Simple option line numbers value is not a number.\n");
	}
      }
      if(l.contains("cbSimpleOptionColored")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  switch(convInt) {
	  case 0:
	    cbSimpleOptionColored->setChecked(false);
	    break;
	  case 1:
	    cbSimpleOptionColored->setChecked(true);
	    break;
	  default:
	    parseErr += tr("Simple option colored value needs to be either one or zero.\n");
	  }
	} else {
	  parseErr += tr("Simple option colored value is not a number.\n");
	}
      }
      if(l.contains("sbSimpleOptionFontSize")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(sbSimpleOptionFontSize->minValue() <= convInt && convInt <= sbSimpleOptionFontSize->maxValue()) {
	    sbSimpleOptionFontSize->setValue(convInt);
	  } else {
	    parseErr += tr("Simple option font size value is out of range.\n");
	  }
	} else {
	  parseErr += tr("Simple option font size value is not a number.\n");
	}
      }

      // Paper tab.
      if(l.contains("cbOrientation")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbOrientation->count() > convInt && convInt >= 0) {
	    cbOrientation->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("Orientation index is out of range.\n");
	  }
	} else {
	  parseErr += tr("Orientation index is not a number.\n");
	}
      }
      if(l.contains("cbFormat")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbFormat->count() > convInt && convInt >= 0) {
	    cbFormat->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("Format index is out of range.\n");
	  }
	} else {
	  parseErr += tr("Format index is not a number.\n");
	}
      }
      if(l.contains("cbDirection")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbDirection->count() > convInt && convInt >= 0) {
	    cbDirection->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("Direction index is out of range.\n");
	  }
	} else {
	  parseErr += tr("Direction index is not a number.\n");
	}
      }
      if(l.contains("cbDuplex")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbDuplex->count() > convInt && convInt >= 0) {
	    cbDuplex->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("Duplex index is out of range.\n");
	  }
	} else {
	  parseErr += tr("Duplex index is not a number.\n");
	}
      }
      if(l.contains("sbRows")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(sbRows->minValue() <= convInt && convInt <= sbRows->maxValue()) {
	    sbRows->setValue(convInt);
	  } else {
	    parseErr += tr("Rows value is out of range.\n");
	  }
	} else {
	  parseErr += tr("Rows value is not a number.\n");
	}
      }
      if(l.contains("sbColumns")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(sbColumns->minValue() <= convInt && convInt <= sbColumns->maxValue()) {
	    sbColumns->setValue(convInt);
	  } else {
	    parseErr += tr("Columns value is out of range.\n");
	  }
	} else {
	  parseErr += tr("Columns value is not a number.\n");
	}
      }
      if(l.contains("sbMargin")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(sbMargin->minValue() <= convInt && convInt <= sbMargin->maxValue()) {
	    sbMargin->setValue(convInt);
	  } else {
	    parseErr += tr("Margin value is out of range.\n");
	  }
	} else {
	  parseErr += tr("Margin value is not a number.\n");
	}
      }
      if(l.contains("cbVirtualBorder")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  switch(convInt) {
	  case 0:
	    cbVirtualBorder->setChecked(false);
	    break;
	  case 1:
	    cbVirtualBorder->setChecked(true);
	    break;
	  default:
	    parseErr += tr("Virtual border value needs to be either one or zero.\n");
	  }
	} else {
	  parseErr += tr("Virtual border value is not a number.\n");
	}
      }

      // Title tab.
      if(l.contains("lePSTitle")) {
	l = l.remove(0, equalPos+1);
	lePSTitle->setText(l);
      }
      if(l.contains("cbHeadersEnable")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  switch(convInt) {
	  case 0:
	    cbHeadersEnable->setChecked(false);
	    break;
	  case 1:
	    cbHeadersEnable->setChecked(true);
	    break;
	  default:
	    parseErr += tr("Headers enable value needs to be either one or zero.\n");
	  }
	} else {
	  parseErr += tr("Headers enable value is not a number.\n");
	}
      }
      if(l.contains("leHeadLeft")) {
	l = l.remove(0, equalPos+1);
	leHeadLeft->setText(l);
      }
      if(l.contains("leHeadCenter")) {
	l = l.remove(0, equalPos+1);
	leHeadCenter->setText(l);
      }
      if(l.contains("leHeadRight")) {
	l = l.remove(0, equalPos+1);
	leHeadRight->setText(l);
      }
      if(l.contains("leWatermark")) {
	l = l.remove(0, equalPos+1);
	leWatermark->setText(l);
      }
      if(l.contains("leFootLeft")) {
	l = l.remove(0, equalPos+1);
	leFootLeft->setText(l);
      }
      if(l.contains("leFootCenter")) {
	l = l.remove(0, equalPos+1);
	leFootCenter->setText(l);
      }
      if(l.contains("leFootRight")) {
	l = l.remove(0, equalPos+1);
	leFootRight->setText(l);
      }

      // Layout tab.
      if(l.contains("slOrder")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(slOrder->minValue() <= convInt && convInt <= slOrder->maxValue()) {
	    slOrder->setValue(convInt);
	  } else {
	    parseErr += tr("Order value is out of range.\n");
	  }
	} else {
	  parseErr += tr("Order value is not a number.\n");
	}
      }
      if(l.contains("sbFontSize")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(sbFontSize->minValue() <= convInt && convInt <= sbFontSize->maxValue()) {
	    sbFontSize->setValue(convInt);
	  } else {
	    parseErr += tr("Font size value is out of range.\n");
	  }
	} else {
	  parseErr += tr("Font size value is not a number.\n");
	}
      }
      if(l.contains("sbRowsPerPage")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(sbRowsPerPage->minValue() <= convInt && convInt <= sbRowsPerPage->maxValue()) {
	    sbRowsPerPage->setValue(convInt);
	  } else {
	    parseErr += tr("Rows per page value is out of range.\n");
	  }
	} else {
	  parseErr += tr("Rows per page value is not a number.\n");
	}
      }
      if(l.contains("sbCharsPerRow")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(sbCharsPerRow->minValue() <= convInt && convInt <= sbCharsPerRow->maxValue()) {
	    sbCharsPerRow->setValue(convInt);
	  } else {
	    parseErr += tr("Chars per row value is out of range.\n");
	  }
	} else {
	  parseErr += tr("Chars per row value is not a number.\n");
	}
      }
      if(l.contains("cbColorLevel")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbColorLevel->count() > convInt && convInt >= 0) {
	    cbColorLevel->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("Color level index is out of range.\n");
	  }
	} else {
	  parseErr += tr("Color level index is not a number.\n");
	}
      }
      if(l.contains("cbComments")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbComments->count() > convInt && convInt >= 0) {
	    cbComments->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("Comments index is out of range.\n");
	  }
	} else {
	  parseErr += tr("Comments index is not a number.\n");
	}
      }
      if(l.contains("cbLanguage")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbLanguage->count() > convInt && convInt >= 0) {
	    cbLanguage->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("Language index is out of range.\n");
	  }
	} else {
	  parseErr += tr("Language index is not a number.\n");
	}
      }
      if(l.contains("cbUnprintables")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbUnprintables->count() > convInt && convInt >= 0) {
	    cbUnprintables->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("Unprintables index is out of range.\n");
	  }
	} else {
	  parseErr += tr("Unprintables index is not a number.\n");
	}
      }
      if(l.contains("cbPostScript")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbPostScript->count() > convInt && convInt >= 0) {
	    cbPostScript->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("PostScript index is out of range.\n");
	  }
	} else {
	  parseErr += tr("PostScript index is not a number.\n");
	}
      }
      if(l.contains("sbTabSize")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(sbTabSize->minValue() <= convInt && convInt <= sbTabSize->maxValue()) {
	    sbTabSize->setValue(convInt);
	  } else {
	    parseErr += tr("Tab size value is out of range.\n");
	  }
	} else {
	  parseErr += tr("Tab size value is not a number.\n");
	}
      }
      if(l.contains("sbLineNumbers")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(sbLineNumbers->minValue() <= convInt && convInt <= sbLineNumbers->maxValue()) {
	    sbLineNumbers->setValue(convInt);
	  } else {
	    parseErr += tr("Line numbers value is out of range.\n");
	  }
	} else {
	  parseErr += tr("Line numbers value is not a number.\n");
	}
      }
      if(l.contains("cbTOC")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  switch(convInt) {
	  case 0:
	    cbTOC->setChecked(false);
	    break;
	  case 1:
	    cbTOC->setChecked(true);
	    break;
	  default:
	    parseErr += tr("TOC value needs to be either one or zero.\n");
	  }
	} else {
	  parseErr += tr("TOC value is not a number.\n");
	}
      }
      if(l.contains("cbTruncate")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  switch(convInt) {
	  case 0:
	    cbTruncate->setChecked(false);
	    break;
	  case 1:
	    cbTruncate->setChecked(true);
	    break;
	  default:
	    parseErr += tr("Truncate value needs to be either one or zero.\n");
	  }
	} else {
	  parseErr += tr("Truncate value is not a number.\n");
	}
      }

      // Processing tab.
      if(l.contains("cbEncoding")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbEncoding->count() > convInt && convInt >= 0) {
	    cbEncoding->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("Encoding index is out of range.\n");
	  }
	} else {
	  parseErr += tr("Encoding index is not a number.\n");
	}
      }
      if(l.contains("cbDelegate")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  switch(convInt) {
	  case 0:
	    cbDelegate->setChecked(false);
	    break;
	  case 1:
	    cbDelegate->setChecked(true);
	    break;
	  default:
	    parseErr += tr("Delegate value needs to be either one or zero.\n");
	  }
	} else {
	  parseErr += tr("Delegate value is not a number.\n");
	}
      }
      if(l.contains("cbPrefeed")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  switch(convInt) {
	  case 0:
	    cbPrefeed->setChecked(false);
	    break;
	  case 1:
	    cbPrefeed->setChecked(true);
	    break;
	  default:
	    parseErr += tr("Prefeed value needs to be either one or zero.\n");
	  }
	} else {
	  parseErr += tr("Prefeed value is not a number.\n");
	}
      }
      if(l.contains("sbCopies")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(sbCopies->minValue() <= convInt && convInt <= sbCopies->maxValue()) {
	    sbCopies->setValue(convInt);
	  } else {
	    parseErr += tr("Copies value is out of range.\n");
	  }
	} else {
	  parseErr += tr("Copies value is not a number.\n");
	}
      }
      if(l.contains("leOptions")) {
	l = l.remove(0, equalPos+1);
	leOptions->setText(l);
      }

      // Tipo tab.
      if(l.contains("cbTipoLanguage")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbTipoLanguage->count() > convInt && convInt >= 0) {
	    cbTipoLanguage->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("Tipo language index is out of range.\n");
	  }
	} else {
	  parseErr += tr("Tipo language index is not a number.\n");
	}
      }
      if(l.contains("cbGUIStyle")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbGUIStyle->count() > convInt && convInt >= 0) {
	    cbGUIStyle->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("GUI style index is out of range.\n");
	  }
	} else {
	  parseErr += tr("GUI style index is not a number.\n");
	}
      }
      if(l.contains("lePathA2ps")) {
	l = l.remove(0, equalPos+1);
	lePathA2ps->setText(l);
      }
      if(l.contains("lePathGv")) {
	l = l.remove(0, equalPos+1);
	lePathGv->setText(l);
      }
      if(l.contains("leConfig")) {
	l = l.remove(0, equalPos+1);
	leConfig->setText(l);
      }
    } // end if contains "="
  } // end while
  
  configFile.close();

  // Check for errors.
  if(parseErr != "") {
    QMessageBox::warning(this,
			 tr("Load configuration"),
			 parseErrIntro + parseErr,
			 QMessageBox::Ok,
			 0);
  }
}

void tipografImpl::saveConfig()
{
  QString configFileComment =
    tr("# Tipograf V" VERSION " configuration file\n"
       "#\n"
       "# This file contains values of all controls related\n"
       "# to printing. It does not contain settings specific\n"
       "# to Tipografs GUI. Those are stored in ~/.tipografrc.\n"
       "# This file is not intended to be edited by hand.\n"
       "#\n"
       "\n");
  
  QFile configFile(leConfig->text());

  // Check config file.
  if(configFile.exists()) {
    if(QMessageBox::information(this,
				tr("Save configuration"),
				tr("You are about to overwrite\n"
				   "an existing file.\n"),
				tr("Continue"), tr("Cancel")) == 1) {
      return;
    }
  }

  if(!configFile.open(IO_WriteOnly)) {
    QMessageBox::warning(this,
			 tr("Save configuration"),
			 tr("Unable to open file for writing."),
			 QMessageBox::Ok,
			 0);
    return;
  }

  QTextStream s(&configFile);

  // Write comment.
  s << configFileComment;

  // Save input / output tab.
  s << tr("\n# Input / Output tab\n");

  s << "leInput="    << leInput->text() << "\n";
  s << "leOutput="   << leInput->text() << "\n";
  s << "cbStdStyle=" << cbStdStyle->currentItem() << "\n";
  s << "cbOutputTo=" << cbOutputTo->currentItem() << "\n";
  s << "cbPrinter="  << cbPrinter->currentItem()  << "\n";
  if(cbSimpleOptionsDisable->isChecked()) {
    s << "cbSimpleOptionsDisable=" << "1" << "\n";
  } else {
    s << "cbSimpleOptionsDisable=" << "0" << "\n";
  }
  if(rbSimpleOptionPortrait->isChecked()) {
    s << "rbSimpleOptionPortrait=" << "1" << "\n";
  } else {
    s << "rbSimpleOptionPortrait=" << "0" << "\n";
  }
  if(rbSimpleOptionLandscape->isChecked()) {
    s << "rbSimpleOptionLandscape=" << "1" << "\n";
  } else {
    s << "rbSimpleOptionLandscape=" << "0" << "\n";
  }
  if(cbSimpleOptionLineNumbers->isChecked()) {
    s << "cbSimpleOptionLineNumbers=" << "1" << "\n";
  } else {
    s << "cbSimpleOptionLineNumbers=" << "0" << "\n";
  }
  if(cbSimpleOptionColored->isChecked()) {
    s << "cbSimpleOptionColored=" << "1" << "\n";
  } else {
    s << "cbSimpleOptionColored=" << "0" << "\n";
  }
  s << "sbSimpleOptionFontSize=" << sbSimpleOptionFontSize->value() << "\n";

  // Save paper tab.
  s << tr("\n# Paper tab\n");

  s << "cbOrientation=" << cbOrientation->currentItem() << "\n";
  s << "cbFormat="      << cbFormat->currentItem()      << "\n";
  s << "cbDirection="   << cbDirection->currentItem()   << "\n";
  s << "cbDuplex="      << cbDuplex->currentItem()      << "\n";
  s << "sbRows="        << sbRows->value()              << "\n";
  s << "sbColumns="     << sbColumns->value()           << "\n";
  s << "sbMargin="      << sbMargin->value()            << "\n";
  if(cbVirtualBorder->isChecked()) {
    s << "cbVirtualBorder=" << "1" << "\n";
  } else {
    s << "cbVirtualBorder=" << "0" << "\n";
  }

  // Save title tab.
  s << tr("\n# Title tab\n");

  s << "lePSTitle=" << lePSTitle->text() << "\n";
  if(cbHeadersEnable->isChecked()) {
    s << "cbHeadersEnable=" << "1" << "\n";
  } else {
    s << "cbHeadersEnable=" << "0" << "\n";
  }
  s << "leHeadLeft="   << leHeadLeft->text()   << "\n";
  s << "leHeadCenter=" << leHeadCenter->text() << "\n";
  s << "leHeadRight="  << leHeadRight->text()  << "\n";
  s << "leWatermark="  << leWatermark->text()  << "\n";
  s << "leFootLeft="   << leFootLeft->text()   << "\n";
  s << "leFootCenter=" << leFootCenter->text() << "\n";
  s << "leFootRight="  << leFootRight->text()  << "\n";
  
  // Save layout tab.
  s << tr("\n# Layout tab\n");

  s << "slOrder="        << slOrder->value()              << "\n";
  s << "sbFontSize="     << sbFontSize->value()           << "\n";
  s << "sbRowsPerPage="  << sbRowsPerPage->value()        << "\n";
  s << "sbCharsPerRow="  << sbCharsPerRow->value()        << "\n";
  s << "cbColorLevel="   << cbColorLevel->currentItem()   << "\n";
  s << "cbComments="     << cbComments->currentItem()     << "\n";
  s << "cbLanguage="     << cbLanguage->currentItem()     << "\n";
  s << "cbUnprintables=" << cbUnprintables->currentItem() << "\n";
  s << "cbPostScript="   << cbPostScript->currentItem()   << "\n";
  s << "sbTabSize="      << sbTabSize->value()            << "\n";
  s << "sbLineNumbers="  << sbLineNumbers->value()        << "\n";
  if(cbTOC->isChecked()) {
    s << "cbTOC=" << "1" << "\n";
  } else {
    s << "cbTOC=" << "0" << "\n";
  }
  if(cbTruncate->isChecked()) {
    s << "cbTruncate=" << "1" << "\n";
  } else {
    s << "cbTruncate=" << "0" << "\n";
  }

  // Save processing tab.
  s << tr("\n# Processing tab\n");

  s << "cbEncoding="   << cbEncoding->currentItem() << "\n";
  if(cbDelegate->isChecked()) {
    s << "cbDelegate=" << "1" << "\n";
  } else {
    s << "cbDelegate=" << "0" << "\n";
  }
  if(cbPrefeed->isChecked()) {
    s << "cbPrefeed=" << "1" << "\n";
  } else {
    s << "cbPrefeed=" << "0" << "\n";
  }
  s << "sbCopies="    << sbCopies->value() << "\n";
  s << "leOptions="   << leOptions->text() << "\n";

  // Save Tipo tab.
  s << tr("\n# Tipo tab\n");

  s << "cbTipoLanguage=" << cbTipoLanguage->currentItem() << "\n";
  s << "cbGUIStyle="     << cbGUIStyle->currentItem()     << "\n";
  s << "lePathA2ps="     << lePathA2ps->text()            << "\n";
  s << "lePathGv="       << lePathGv->text()              << "\n";
  s << "leConfig="       << leConfig->text()              << "\n";
  
  // Close config file.
  configFile.close();

}

// ============================================================
// Tipo load/save
// These functions do the I/O to ~/.tipografrc.
// They only save/load the Tipo tab values 
//     - TipoLanguage
//     - GUIStyle
//     - PathA2ps
//     - PathGv
// ============================================================

void tipografImpl::loadTipo()
{
  // Prepare file and error objects.
  QFile configFile(resolveConfigFileName());
  QString parseErrIntro = tr("The following errors occured:\n\n");
  QString parseErr      = "";

  // Check config file.
  if(!configFile.exists()) {
    QMessageBox::warning(this,
			 tr("Load Tipo configuration"),
			 tr("The configuration file ~/.tipografrc\n"
			    "does not exist."),
			 QMessageBox::Ok,
			 0);
    return;
  }
  
  if(!configFile.open(IO_ReadOnly)) {
    QMessageBox::warning(this,
			 tr("Load Tipo configuration"),
			 tr("Unable to open file ~/.tipografrc for reading."),
			 QMessageBox::Ok,
			 0);
    return;
  }
  
  // Read in configuration.
  QTextStream s(&configFile);
  QString l;
  int equalPos = 0;
  int convInt = 0;
  bool convOk = false;

  while(!s.eof()) {
    l = s.readLine();
    if(l.contains("=")) {
      equalPos = l.find("=");

      // Tipo tab.
      if(l.contains("cbTipoLanguage")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbTipoLanguage->count() > convInt && convInt >= 0) {
	    cbTipoLanguage->setCurrentItem(convInt);
	  } else {
	    parseErr += tr("Tipo language index is out of range.\n");
	  }
	} else {
	  parseErr += tr("Tipo language index is not a number.\n");
	}
      }
      if(l.contains("cbGUIStyle")) {
	l = l.remove(0, equalPos+1);
	convInt = l.toInt(&convOk, 10);
	if(convOk == true) {
	  if(cbGUIStyle->count() > convInt && convInt >= 0) {
	    cbGUIStyle->setCurrentItem(convInt);
	    QApplication::setStyle(QStyleFactory::create(cbGUIStyle->currentText()));
	  } else {
	    parseErr += tr("GUI style index is out of range.\n");
	  }
	} else {
	  parseErr += tr("GUI style index is not a number.\n");
	}
      }
      if(l.contains("lePathA2ps")) {
	l = l.remove(0, equalPos+1);
	lePathA2ps->setText(l);
      }
      if(l.contains("lePathGv")) {
	l = l.remove(0, equalPos+1);
	lePathGv->setText(l);
      }
    } // end if contains "="
  } // end while
  
  configFile.close();

  // Check for errors.
  if(parseErr != "") {
    QMessageBox::warning(this,
			 tr("Load Tipo configuration"),
			 parseErrIntro + parseErr,
			 QMessageBox::Ok,
			 0);
    return;
  }

}

void tipografImpl::saveTipo()
{
  QString configFileComment =
    tr("# Tipograf V" VERSION " configuration file\n"
       "#\n"
       "# This file only contains settings related to\n"
       "# Tipograf's GUI.\n"
       "# This file is not intended to be edited by hand.\n"
       "#\n"
       "\n");
  
  QFile configFile(resolveConfigFileName());

  // Check config file.
  if(!configFile.open(IO_WriteOnly)) {
    QMessageBox::warning(this,
			 tr("Save Tipo configuration"),
			 tr("Unable to open file for writing."),
			 QMessageBox::Ok,
			 0);
    return;
  }

  QTextStream s(&configFile);

  // Write comment.
  s << configFileComment;

  // Save Tipo tab.
  s << tr("\n# Tipo tab\n");

  s << "cbTipoLanguage=" << cbTipoLanguage->currentItem() << "\n";
  s << "cbGUIStyle="     << cbGUIStyle->currentItem()     << "\n";
  s << "lePathA2ps="     << lePathA2ps->text()            << "\n";
  s << "lePathGv="       << lePathGv->text()              << "\n";
  
  // Close config file.
  configFile.close();

}
