#include <stdlib.h>
#include <stdio.h>

#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qslider.h>
#include <qgroupbox.h>
#include <qtabwidget.h>
#include <qtextbrowser.h>
#include <qbuttongroup.h>
#include <qstylefactory.h>

#include "tipografImpl.h"
#include "init.h"

// ============================================================
// Constructor
// ============================================================
tipografImpl::tipografImpl(tipoInit * tiInitVals,
			   QWidget* parent,
			   const char* name,
			   bool modal, WFlags fl)
  : dlgTipograf( parent, name, modal, fl )
{
  finishedStartUp = false;

  // Default Style
  slStdStyleOpt += tr("");
  slStdStyleOpt += tr("");
  slStdStyleOpt += tr("-1");
  slStdStyleOpt += tr("-2");
  slStdStyleOpt += tr("-3");
  slStdStyleOpt += tr("-4");
  slStdStyleOpt += tr("-5");
  slStdStyleOpt += tr("-6");
  slStdStyleOpt += tr("-7");
  slStdStyleOpt += tr("-8");
  slStdStyleOpt += tr("-9");

  slStdStyleTxt += tr("auto");
  slStdStyleTxt += tr("self defined");
  slStdStyleTxt += tr("1x1 portrait");
  slStdStyleTxt += tr("2x1 landscape");
  slStdStyleTxt += tr("3x1 landscape");
  slStdStyleTxt += tr("2x2 portrait");
  slStdStyleTxt += tr("5x1 landscape");
  slStdStyleTxt += tr("3x2 landscape");
  slStdStyleTxt += tr("7x1 landscape");
  slStdStyleTxt += tr("4x2 landscape");
  slStdStyleTxt += tr("3x3 landscape");

  cbStdStyle->insertStringList(slStdStyleTxt);
  selfDefinedStyle();

  // OutputTo
  slOutputToTxt += tr("printer");
  slOutputToTxt += tr("file");

  sOutputToFileOpt = tr("--output=");

  cbOutputTo->insertStringList(slOutputToTxt);

  // Printer
  slPrinterOpt += tr("-d");
#ifdef TU_BERLIN
  slPrinterOpt += tr("--printer=lp");
  slPrinterOpt += tr("--printer=en210");
  slPrinterOpt += tr("--printer=en211");
  slPrinterOpt += tr("--printer=en212");
  slPrinterOpt += tr("--printer=en235");
  slPrinterOpt += tr("--printer=en237");
  slPrinterOpt += tr("--printer=en704");
  slPrinterOpt += tr("--printer=en704mv");
  slPrinterOpt += tr("--printer=en706c");
  slPrinterOpt += tr("--printer=en732-2");
  slPrinterOpt += tr("--printer=en732");
  slPrinterOpt += tr("--printer=fr2517");
  slPrinterOpt += tr("--printer=fr2518");
  slPrinterOpt += tr("--printer=fr2521c");
  slPrinterOpt += tr("--printer=fr2521e");
  slPrinterOpt += tr("--printer=fr2521");
  slPrinterOpt += tr("--printer=fr3039");
  slPrinterOpt += tr("--printer=fr3041");
  slPrinterOpt += tr("--printer=fr3047");
  slPrinterOpt += tr("--printer=fr3514a");
  slPrinterOpt += tr("--printer=fr3514");
  slPrinterOpt += tr("--printer=fr5027");
  slPrinterOpt += tr("--printer=fr5030");
  slPrinterOpt += tr("--printer=fr5032");
  slPrinterOpt += tr("--printer=fr5038");
  slPrinterOpt += tr("--printer=fr5041c");
  slPrinterOpt += tr("--printer=fr5041swt");
  slPrinterOpt += tr("--printer=fr5041uebb");
  slPrinterOpt += tr("--printer=fr5055");
  slPrinterOpt += tr("--printer=fr5060");
  slPrinterOpt += tr("--printer=fr5070");
  slPrinterOpt += tr("--printer=fr5084");
  slPrinterOpt += tr("--printer=fr5087c");
  slPrinterOpt += tr("--printer=fr5091");
  slPrinterOpt += tr("--printer=fr5503");
  slPrinterOpt += tr("--printer=fr5507");
  slPrinterOpt += tr("--printer=fr5514");
  slPrinterOpt += tr("--printer=fr5531");
  slPrinterOpt += tr("--printer=fr5531-o");
  slPrinterOpt += tr("--printer=fr5535");
  slPrinterOpt += tr("--printer=fr5539");
  slPrinterOpt += tr("--printer=fr6005");
  slPrinterOpt += tr("--printer=fr6015e");
  slPrinterOpt += tr("--printer=fr6015");
  slPrinterOpt += tr("--printer=fr6021");
  slPrinterOpt += tr("--printer=fr6025");
  slPrinterOpt += tr("--printer=fr6030");
  slPrinterOpt += tr("--printer=fr6032");
  slPrinterOpt += tr("--printer=fr6036");
  slPrinterOpt += tr("--printer=fr6039");
  slPrinterOpt += tr("--printer=fr6041");
  slPrinterOpt += tr("--printer=fr6043");
  slPrinterOpt += tr("--printer=fr6047c");
  slPrinterOpt += tr("--printer=fr6047e");
  slPrinterOpt += tr("--printer=fr6047");
  slPrinterOpt += tr("--printer=fr6068");
  slPrinterOpt += tr("--printer=fr6076c");
  slPrinterOpt += tr("--printer=fr6079");
  slPrinterOpt += tr("--printer=fr6514");
  slPrinterOpt += tr("--printer=fr6519");
  slPrinterOpt += tr("--printer=fr6536");
  slPrinterOpt += tr("--printer=gor008c");
  slPrinterOpt += tr("--printer=gor008");
  slPrinterOpt += tr("--printer=gor102c");
  slPrinterOpt += tr("--printer=gor102");
  slPrinterOpt += tr("--printer=ma210");
  slPrinterOpt += tr("--printer=tel1012");
#endif

  slPrinterTxt += tr("auto");
#ifdef TU_BERLIN
  slPrinterTxt += tr("lp");
  slPrinterTxt += tr("en210");
  slPrinterTxt += tr("en211");
  slPrinterTxt += tr("en212");
  slPrinterTxt += tr("en235");
  slPrinterTxt += tr("en237");
  slPrinterTxt += tr("en704");
  slPrinterTxt += tr("en704mv");
  slPrinterTxt += tr("en706c");
  slPrinterTxt += tr("en732-2");
  slPrinterTxt += tr("en732");
  slPrinterTxt += tr("fr2517");
  slPrinterTxt += tr("fr2518");
  slPrinterTxt += tr("fr2521c");
  slPrinterTxt += tr("fr2521e");
  slPrinterTxt += tr("fr2521");
  slPrinterTxt += tr("fr3039");
  slPrinterTxt += tr("fr3041");
  slPrinterTxt += tr("fr3047");
  slPrinterTxt += tr("fr3514a");
  slPrinterTxt += tr("fr3514");
  slPrinterTxt += tr("fr5027");
  slPrinterTxt += tr("fr5030");
  slPrinterTxt += tr("fr5032");
  slPrinterTxt += tr("fr5038");
  slPrinterTxt += tr("fr5041c");
  slPrinterTxt += tr("fr5041swt");
  slPrinterTxt += tr("fr5041uebb");
  slPrinterTxt += tr("fr5055");
  slPrinterTxt += tr("fr5060");
  slPrinterTxt += tr("fr5070");
  slPrinterTxt += tr("fr5084");
  slPrinterTxt += tr("fr5087c");
  slPrinterTxt += tr("fr5091");
  slPrinterTxt += tr("fr5503");
  slPrinterTxt += tr("fr5507");
  slPrinterTxt += tr("fr5514");
  slPrinterTxt += tr("fr5531");
  slPrinterTxt += tr("fr5531-o");
  slPrinterTxt += tr("fr5535");
  slPrinterTxt += tr("fr5539");
  slPrinterTxt += tr("fr6005");
  slPrinterTxt += tr("fr6015e");
  slPrinterTxt += tr("fr6015");
  slPrinterTxt += tr("fr6021");
  slPrinterTxt += tr("fr6025");
  slPrinterTxt += tr("fr6030");
  slPrinterTxt += tr("fr6032");
  slPrinterTxt += tr("fr6036");
  slPrinterTxt += tr("fr6039");
  slPrinterTxt += tr("fr6041");
  slPrinterTxt += tr("fr6043");
  slPrinterTxt += tr("fr6047c");
  slPrinterTxt += tr("fr6047e");
  slPrinterTxt += tr("fr6047");
  slPrinterTxt += tr("fr6068");
  slPrinterTxt += tr("fr6076c");
  slPrinterTxt += tr("fr6079");
  slPrinterTxt += tr("fr6514");
  slPrinterTxt += tr("fr6519");
  slPrinterTxt += tr("fr6536");
  slPrinterTxt += tr("gor008c");
  slPrinterTxt += tr("gor008");
  slPrinterTxt += tr("gor102c");
  slPrinterTxt += tr("gor102");
  slPrinterTxt += tr("ma210");
  slPrinterTxt += tr("tel1012");
#endif
 
  slPrinterDesc += tr("default printer");
#ifdef TU_BERLIN
  slPrinterDesc += tr("lp - Drucker zum irrtmlichen Ausdrucken");
  slPrinterDesc += tr("en210 - HP LaserJet 4050N im EN210");
  slPrinterDesc += tr("en211 - HP Color LaserJet 4500 im EN211");
  slPrinterDesc += tr("en212 - HP LaserJet 4M Plus im EN212");
  slPrinterDesc += tr("en235 - HP LaserJet 2100N im EN235");
  slPrinterDesc += tr("en237 - HP LaserJet 2100N im EN237");
  slPrinterDesc += tr("en704 - HPLaserJet4100N des FG CIS im EN 704");
  slPrinterDesc += tr("en704mv - HP LaserJet 4MV des FG CIS im EN704");
  slPrinterDesc += tr("en706c - XeroxDocuprint NC60 des FG CIS im EN706");
  slPrinterDesc += tr("en732-2 - HP LaserJet 4M im EN732");
  slPrinterDesc += tr("en732 - en705 - HP LaserJet 4M im EN732");
  slPrinterDesc += tr("fr2517 - HPLaserJet4500N des FG PDV im FR 2517");
  slPrinterDesc += tr("fr2518 - HP LaserJet 4Si im FR2518");
  slPrinterDesc += tr("fr2521c - Minolta Magicolor 2200 des FG NI im FR2521");
  slPrinterDesc += tr("fr2521e - HP LaserJet 4 Plus des FG NI im FR2521");
  slPrinterDesc += tr("fr2521 - HP LaserJet 4 Plus des FG NI im FR2521");
  slPrinterDesc += tr("fr3039 - rtpcs - PCS/Fujitsu Matrixdrucker des FG RT im FR3039");
  slPrinterDesc += tr("fr3041 - rthplj - HP Laserjet 3 der Gruppe rt im FR 3041");
  slPrinterDesc += tr("fr3047 - HP Laserjet 4050NT des FG ROSW im FR3047");
  slPrinterDesc += tr("fr3514a - HP LaserJet 4Si im FR3514a");
  slPrinterDesc += tr("fr3514 - microline 83a Matrixdrucker des FG ROSW im FR fr3514");
  slPrinterDesc += tr("fr5027 - HP OfficeJet Pro1150C des FG UEBB im FR 5027");
  slPrinterDesc += tr("fr5030 - HP LaserJet 4 M Plus des FG UEBB im FR 5030");
  slPrinterDesc += tr("fr5032 - HP LaserJet 4 M Plus des FG SWT im FR 5032");
  slPrinterDesc += tr("fr5038 - HP DescJet 2100 des FG PVP im FR 5038");
  slPrinterDesc += tr("fr5041c - HP550C des FG SWT im FR 5041");
  slPrinterDesc += tr("fr5041swt - HP Laserjet 4050 des FG SWT im FR 5041");
  slPrinterDesc += tr("fr5041uebb - HPLaserJet4500N des FG UEBB im FR 5041");
  slPrinterDesc += tr("fr5055 - HP Laserjet 4m des FG KI im FR 5055");
  slPrinterDesc += tr("fr5060 - HP Laserjet 4m des FG WBS im FR 5060");
  slPrinterDesc += tr("fr5070 - HP LaserJet 4 Plus der Fachbereichsverwaltung");
  slPrinterDesc += tr("fr5084 - fr5085 - HP LaserJet 8000 im FR5084");
  slPrinterDesc += tr("fr5087c - Tektronix Phaser 560 im FR 5087");
  slPrinterDesc += tr("fr5091 - HP LaserJet 4Si im Sekretariat IRB im FR5091");
  slPrinterDesc += tr("fr5503 - HP LaserJet 4050 des FG IG im FR 5503");
  slPrinterDesc += tr("fr5507 - HP LaserJet 4 M Plus des FG INI im FR 5507");
  slPrinterDesc += tr("fr5514 - HP LaserJet 4Si im FR5514");
  slPrinterDesc += tr("fr5531 - Kyocera FS-3700 des FG AEDV im FR5531");
  slPrinterDesc += tr("fr5531-o - HP Laserjet 4M des FG AEDV im FR 5531");
  slPrinterDesc += tr("fr5535 - HP LaserJet 5SiMX im FR5535");
  slPrinterDesc += tr("fr5539 - HP LaserJet4000 im FR5539");
  slPrinterDesc += tr("fr6005 - HP LaserJet4000 des FG OEK im FR6005");
  slPrinterDesc += tr("fr6015e - fr6076e - HP Laserjet 4m des FG FLP im FR 6015");
  slPrinterDesc += tr("fr6015 - fr6076 - HP Laserjet 4m des FG FLP im FR 6015");
  slPrinterDesc += tr("fr6021 - HP LaserJet 4050 des FG TAL im FR 6021");
  slPrinterDesc += tr("fr6025 - HP Laserjet 5 des FG TFS im FR 6025");
  slPrinterDesc += tr("fr6030 - HP Laserjet 4100 des FG KBS im FR6030");
  slPrinterDesc += tr("fr6032 - fr6028 - HP Laserjet 4m des FG KBS im FR6032");
  slPrinterDesc += tr("fr6036 - HP Laserjet 4550 des FG KBS im FR6036");
  slPrinterDesc += tr("fr6039 - HP Laserjet des FG TFS im FR6039");
  slPrinterDesc += tr("fr6041 - Apple Laserwriter Pro des FG TAL im FR 6041");
  slPrinterDesc += tr("fr6043 - fr6044 - HP LaserJet4000 im FR6043");
  slPrinterDesc += tr("fr6047c - fr6045 - HP DescJet 1600CM des FG STAT im FR 6047");
  slPrinterDesc += tr("fr6047e - HP Laserjet 4000 des FG STAT im FR 6047");
  slPrinterDesc += tr("fr6047 - fr6518 - HP Laserjet 4000 des FG STAT im FR 6047");
  slPrinterDesc += tr("fr6068 - HP Laserjet 5m des FG FLP im FR 6068");
  slPrinterDesc += tr("fr6076c - Tektronix Phaser 750DP des FG FLP im FR 6076");
  slPrinterDesc += tr("fr6079 - HP DescJet 1600CM des FG FLP im FR 6079");
  slPrinterDesc += tr("fr6514 - HP LaserJet 4Si im FR6514");
  slPrinterDesc += tr("fr6519 - HP LaserJet 4050N im FR6519");
  slPrinterDesc += tr("fr6536 - HP LaserJet4000 im FR6536");
  slPrinterDesc += tr("gor008c - gor103 - Xerox Xprint4925 des DAI-Labors im GOR 008");
  slPrinterDesc += tr("gor008 - HP Laserjet 5Si des DAI-Labors im GOR 008");
  slPrinterDesc += tr("gor102c - HPLaserJet 4000 des DAI-Labors im GOR 008");
  slPrinterDesc += tr("gor102 - HPLaserJet 4000 des DAI-Labors im GOR 008");
  slPrinterDesc += tr("ma210 - HP LaserJet 4si im MA218");
  slPrinterDesc += tr("tel1012 - HP LaserJet 4M Plus des FG APA im TEL1012");
#endif

  cbPrinter->insertStringList(slPrinterTxt);
  printer();

  // Type
  sTypeOpt = tr("--guess");

  // Simple Options
  sSimpleOptionLineNumbersOpt = tr("--line-numbers=5");
  sSimpleOptionColoredOpt = tr("--highlight-level=heavy");

  simpleOptions();


  // Format
  slFormatTxt += tr("auto");
  slFormatTxt += tr("10x14");
  slFormatTxt += tr("A3");
  slFormatTxt += tr("A4");
  slFormatTxt += tr("A4dj");
  slFormatTxt += tr("A5");
  slFormatTxt += tr("B4");
  slFormatTxt += tr("B5");
  slFormatTxt += tr("Executive");
  slFormatTxt += tr("Folio");
  slFormatTxt += tr("Ledger");
  slFormatTxt += tr("Legal");
  slFormatTxt += tr("Letter");
  slFormatTxt += tr("Letterdj");
  slFormatTxt += tr("Quarto");
  slFormatTxt += tr("Statement");
  slFormatTxt += tr("Tabloid");

  slFormatOpt += tr("");
  slFormatOpt += tr("--medium=10x14");
  slFormatOpt += tr("--medium=A3");
  slFormatOpt += tr("--medium=A4");
  slFormatOpt += tr("--medium=A4dj");
  slFormatOpt += tr("--medium=A5");
  slFormatOpt += tr("--medium=B4");
  slFormatOpt += tr("--medium=B5");
  slFormatOpt += tr("--medium=Executive");
  slFormatOpt += tr("--medium=Folio");
  slFormatOpt += tr("--medium=Ledger");
  slFormatOpt += tr("--medium=Legal");
  slFormatOpt += tr("--medium=Letter");
  slFormatOpt += tr("--medium=Letterdj");
  slFormatOpt += tr("--medium=Quarto");
  slFormatOpt += tr("--medium=Statement");
  slFormatOpt += tr("--medium=Tabloid");
  
  cbFormat->insertStringList(slFormatTxt);

  // Orientation
  slOrientationOpt += tr("");
  slOrientationOpt += tr("--portrait");
  slOrientationOpt += tr("--landscape");

  slOrientationTxt += tr("auto");
  slOrientationTxt += tr("portrait");
  slOrientationTxt += tr("landscape");

  cbOrientation->insertStringList(slOrientationTxt);

  // Direction
  slDirectionOpt += tr("");
  slDirectionOpt += tr("--major=rows");
  slDirectionOpt += tr("--major=columns");

  slDirectionTxt += tr("auto");
  slDirectionTxt += tr("lines first");
  slDirectionTxt += tr("columns first");

  cbDirection->insertStringList(slDirectionTxt);

  // Rows
  sRowsOpt = tr("--rows=");

  // Columns
  sColumnsOpt = tr("--columns=");

  // Margin
  sMarginOpt = tr("--margin=");

  // Duplex
  slDuplexOpt += tr("");
  slDuplexOpt += tr("--sides=simplex");
  slDuplexOpt += tr("--sides=duplex");
  slDuplexOpt += tr("--sides=tumble");

  slDuplexTxt += tr("auto");
  slDuplexTxt += tr("single page");
  slDuplexTxt += tr("double page");
  slDuplexTxt += tr("2*1 double page");

  cbDuplex->insertStringList(slDuplexTxt);

  // Virtual Borders
  sVirtualBorderOpt = tr("--borders");

  // PSTitle
  lePSTitle->setText(tr("Generated by Tipograf Version 1.0"));
  sPSTitleOpt = tr("--title=");

  // Headings
  sHeadLeftOpt   = tr("--left-title=");
  sHeadCenterOpt = tr("--center-title=");
  sHeadRightOpt  = tr("--right-title=");
  sFootLeftOpt   = tr("--left-footer=");
  sFootCenterOpt = tr("--footer=");
  sFootRightOpt  = tr("--right-footer=");
  sWatermarkOpt  = tr("--underlay=");

  leHeadLeft->setText(tr("#?2||$e $T|"));
  leHeadCenter->setText(tr("#?1|$t1|$n|"));
  leHeadRight->setText(tr("#?2|$t2|$Q|"));
  leFootLeft->setText(tr("#?l!%E!#?v|%E|%s./%s#|!"));
  leFootCenter->setText(tr("#?l|#!s-$f-, -||"));
  leFootRight->setText(tr("#?l!%s./%s#!#?v|%s./%s#|%E|!"));

  cbHeadersEnable->setChecked(false);
  selfDefinedHeaders();

  // Order
  slOrderTxt += tr("Font, Lines, Chars");
  slOrderTxt += tr("Font, Chars, Lines");
  slOrderTxt += tr("Lines, Font, Chars");
  slOrderTxt += tr("Lines, Chars, Font");
  slOrderTxt += tr("Chars, Font, Lines");
  slOrderTxt += tr("Chars, Lines, Font");

  tlOrder->setText(*(slOrderTxt.begin()));
  iOrder = 0;

  // FontSize
  sFontSizeOpt = tr("--font-size=");

  // RowsPerPage
  sRowsPerPageOpt = tr("--lines-per-page=");

  // CharsPerRow
  sCharsPerRowOpt =tr("--chars-per-line=");

  // ColorLevel
  slColorLevelOpt += tr("");
  slColorLevelOpt += tr("--highlight-level=none");
  slColorLevelOpt += tr("--highlight-level=normal");
  slColorLevelOpt += tr("--highlight-level=heavy");

  slColorLevelTxt += tr("auto");
  slColorLevelTxt += tr("none");
  slColorLevelTxt += tr("normal");
  slColorLevelTxt += tr("heavy");
  
  cbColorLevel->insertStringList(slColorLevelTxt);

  // Comments
  slCommentsOpt += tr("");
  slCommentsOpt += tr("--strip-level=0");
  slCommentsOpt += tr("--strip-level=1");
  slCommentsOpt += tr("--strip-level=2");
  slCommentsOpt += tr("--strip-level=3");

  slCommentsTxt += tr("auto");
  slCommentsTxt += tr("print all");
  slCommentsTxt += tr("no normal comments");
  slCommentsTxt += tr("no strong comments");
  slCommentsTxt += tr("no comments at all");

  cbComments->insertStringList(slCommentsTxt);

  // Language
  slLanguageOpt += tr("");
  slLanguageOpt += tr("--pretty-print=plain");
  slLanguageOpt += tr("--pretty-print=68000");
  slLanguageOpt += tr("--pretty-print=a2psrc");
  slLanguageOpt += tr("--pretty-print=ssh");
  slLanguageOpt += tr("--pretty-print=ada");
  slLanguageOpt += tr("--pretty-print=asn1");
  slLanguageOpt += tr("--pretty-print=autoconf");
  slLanguageOpt += tr("--pretty-print=awk");
  slLanguageOpt += tr("--pretty-print=b");
  slLanguageOpt += tr("--pretty-print=bc");
  slLanguageOpt += tr("--pretty-print=sh");
  slLanguageOpt += tr("--pretty-print=c");
  slLanguageOpt += tr("--pretty-print=csh");
  slLanguageOpt += tr("--pretty-print=cxx");
  slLanguageOpt += tr("--pretty-print=chlog");
  slLanguageOpt += tr("--pretty-print=claire");
  slLanguageOpt += tr("--pretty-print=clisp");
  slLanguageOpt += tr("--pretty-print=coqv");
  slLanguageOpt += tr("--pretty-print=cidl");
  slLanguageOpt += tr("--pretty-print=cpp");
  slLanguageOpt += tr("--pretty-print=dc_shell");
  slLanguageOpt += tr("--pretty-print=eiffel");
  slLanguageOpt += tr("--pretty-print=elisp");
  slLanguageOpt += tr("--pretty-print=eps");
  slLanguageOpt += tr("--pretty-print=tclx");
  slLanguageOpt += tr("--pretty-print=fortran");
  slLanguageOpt += tr("--pretty-print=for77-fixed");
  slLanguageOpt += tr("--pretty-print=for77-free");
  slLanguageOpt += tr("--pretty-print=for77kwds");
  slLanguageOpt += tr("--pretty-print=for90-fixed");
  slLanguageOpt += tr("--pretty-print=for90-free");
  slLanguageOpt += tr("--pretty-print=for90-kwds");
  slLanguageOpt += tr("--pretty-print=for-fixed");
  slLanguageOpt += tr("--pretty-print=for-free");
  slLanguageOpt += tr("--pretty-print=gnuc");
  slLanguageOpt += tr("--pretty-print=gmake");
  slLanguageOpt += tr("--pretty-print=haskell");
  slLanguageOpt += tr("--pretty-print=html");
  slLanguageOpt += tr("--pretty-print=idl");
  slLanguageOpt += tr("--pretty-print=is5rul");
  slLanguageOpt += tr("--pretty-print=java");
  slLanguageOpt += tr("--pretty-print=js");
  slLanguageOpt += tr("--pretty-print=lace");
  slLanguageOpt += tr("--pretty-print=lex");
  slLanguageOpt += tr("--pretty-print=lout");
  slLanguageOpt += tr("--pretty-print=mail");
  slLanguageOpt += tr("--pretty-print=make");
  slLanguageOpt += tr("--pretty-print=mib");
  slLanguageOpt += tr("--pretty-print=maple");
  slLanguageOpt += tr("--pretty-print=matlab4");
  slLanguageOpt += tr("--pretty-print=modula2");
  slLanguageOpt += tr("--pretty-print=modula3");
  slLanguageOpt += tr("--pretty-print=o2c");
  slLanguageOpt += tr("--pretty-print=oberon");
  slLanguageOpt += tr("--pretty-print=objc");
  slLanguageOpt += tr("--pretty-print=ocaml");
  slLanguageOpt += tr("--pretty-print=mly");
  slLanguageOpt += tr("--pretty-print=octave");
  slLanguageOpt += tr("--pretty-print=initora");
  slLanguageOpt += tr("--pretty-print=plsql");
  slLanguageOpt += tr("--pretty-print=sql");
  slLanguageOpt += tr("--pretty-print=oracle");
  slLanguageOpt += tr("--pretty-print=pascal");
  slLanguageOpt += tr("--pretty-print=perl");
  slLanguageOpt += tr("--pretty-print=ps");
  slLanguageOpt += tr("--pretty-print=ppd");
  slLanguageOpt += tr("--pretty-print=pov");
  slLanguageOpt += tr("--pretty-print=pre");
  slLanguageOpt += tr("--pretty-print=pretex");
  slLanguageOpt += tr("--pretty-print=prolog");
  slLanguageOpt += tr("--pretty-print=promela");
  slLanguageOpt += tr("--pretty-print=python");
  slLanguageOpt += tr("--pretty-print=card");
  slLanguageOpt += tr("--pretty-print=rexx");
  slLanguageOpt += tr("--pretty-print=sather");
  slLanguageOpt += tr("--pretty-print=scheme");
  slLanguageOpt += tr("--pretty-print=sdl88");
  slLanguageOpt += tr("--pretty-print=sed");
  slLanguageOpt += tr("--pretty-print=sql92");
  slLanguageOpt += tr("--pretty-print=sml");
  slLanguageOpt += tr("--pretty-print=tcsh");
  slLanguageOpt += tr("--pretty-print=tex");
  slLanguageOpt += tr("--pretty-print=texinfo");
  slLanguageOpt += tr("--pretty-print=texscript");
  slLanguageOpt += tr("--pretty-print=tiger");
  slLanguageOpt += tr("--pretty-print=tk");
  slLanguageOpt += tr("--pretty-print=tcl");
  slLanguageOpt += tr("--pretty-print=udiff");
  slLanguageOpt += tr("--pretty-print=unity");
  slLanguageOpt += tr("--pretty-print=verilog");
  slLanguageOpt += tr("--pretty-print=vhdl");
  slLanguageOpt += tr("--pretty-print=vba");
  slLanguageOpt += tr("--pretty-print=vtcl");
  slLanguageOpt += tr("--pretty-print=vrml");
  slLanguageOpt += tr("--pretty-print=wdiff");
  slLanguageOpt += tr("--pretty-print=xs");
  slLanguageOpt += tr("--pretty-print=yacc");
  slLanguageOpt += tr("--pretty-print=zsh");

  slLanguageTxt += tr("auto");
  slLanguageTxt += tr("none");
  slLanguageTxt += tr("68k Assembler");
  slLanguageTxt += tr("a2ps config");
  slLanguageTxt += tr("a2ps style");
  slLanguageTxt += tr("Ada");
  slLanguageTxt += tr("ASN.1");
  slLanguageTxt += tr("Autoconf");
  slLanguageTxt += tr("AWK");
  slLanguageTxt += tr("B");
  slLanguageTxt += tr("BC");
  slLanguageTxt += tr("Bash");
  slLanguageTxt += tr("C");
  slLanguageTxt += tr("C Shell");
  slLanguageTxt += tr("C++");
  slLanguageTxt += tr("Changelog");
  slLanguageTxt += tr("Claire");
  slLanguageTxt += tr("Common Lisp");
  slLanguageTxt += tr("Coq Vernacular");
  slLanguageTxt += tr("CORBA IDL");
  slLanguageTxt += tr("CPP");
  slLanguageTxt += tr("DC Shell");
  slLanguageTxt += tr("Eiffel");
  slLanguageTxt += tr("Emacs Lisp");
  slLanguageTxt += tr("EPS");
  slLanguageTxt += tr("Extended TCL");
  slLanguageTxt += tr("Fortran");
  slLanguageTxt += tr("Fortran 77 Fixed");
  slLanguageTxt += tr("Fortran 77 Free");
  slLanguageTxt += tr("Fortran 77 Keywords");
  slLanguageTxt += tr("Fortran 90 Fixed");
  slLanguageTxt += tr("Fortran 90 Free");
  slLanguageTxt += tr("Fortran 90 Keywords");
  slLanguageTxt += tr("Fortran Fixed");
  slLanguageTxt += tr("Fortran Free");
  slLanguageTxt += tr("GNUish C");
  slLanguageTxt += tr("GNUMakefile");
  slLanguageTxt += tr("Haskell");
  slLanguageTxt += tr("HTML");
  slLanguageTxt += tr("IDL");
  slLanguageTxt += tr("InstallShield 5");
  slLanguageTxt += tr("Java");
  slLanguageTxt += tr("JavaScript");
  slLanguageTxt += tr("LACE");
  slLanguageTxt += tr("Lex");
  slLanguageTxt += tr("Lout");
  slLanguageTxt += tr("Mail Ordner");
  slLanguageTxt += tr("Makefile");
  slLanguageTxt += tr("Management Info Base (MIB)");
  slLanguageTxt += tr("Maple");
  slLanguageTxt += tr("MATLAB 4");
  slLanguageTxt += tr("Modula 2");
  slLanguageTxt += tr("Modula 3");
  slLanguageTxt += tr("o2c");
  slLanguageTxt += tr("Oberon");
  slLanguageTxt += tr("Objective C");
  slLanguageTxt += tr("OCaml");
  slLanguageTxt += tr("OCaml Yacc");
  slLanguageTxt += tr("Octave");
  slLanguageTxt += tr("Oracle parameter file");
  slLanguageTxt += tr("Oracle PL/SQL");
  slLanguageTxt += tr("Oracle SQL");
  slLanguageTxt += tr("Oracle SQL-PL/SQL-SQL*Plus");
  slLanguageTxt += tr("Pascal");
  slLanguageTxt += tr("Perl");
  slLanguageTxt += tr("PostScript");
  slLanguageTxt += tr("PS printer description (PPD)");
  slLanguageTxt += tr("Pov-Ray");
  slLanguageTxt += tr("PreScript");
  slLanguageTxt += tr("PreTeX");
  slLanguageTxt += tr("Prolog");
  slLanguageTxt += tr("Promela");
  slLanguageTxt += tr("Python");
  slLanguageTxt += tr("reference card");
  slLanguageTxt += tr("REXX");
  slLanguageTxt += tr("Sather");
  slLanguageTxt += tr("Scheme");
  slLanguageTxt += tr("SDL-88");
  slLanguageTxt += tr("Sed");
  slLanguageTxt += tr("SQL 92");
  slLanguageTxt += tr("Standard ML");
  slLanguageTxt += tr("TC Shell");
  slLanguageTxt += tr("TeX");
  slLanguageTxt += tr("Texinfo");
  slLanguageTxt += tr("TeXScript");
  slLanguageTxt += tr("Tiger");
  slLanguageTxt += tr("TK");
  slLanguageTxt += tr("TCL");
  slLanguageTxt += tr("Unified Diff");
  slLanguageTxt += tr("Unity");
  slLanguageTxt += tr("VERILOG");
  slLanguageTxt += tr("VHDL");
  slLanguageTxt += tr("VBA");
  slLanguageTxt += tr("Visual TCL");
  slLanguageTxt += tr("VRML");
  slLanguageTxt += tr("wdiff");
  slLanguageTxt += tr("XS");
  slLanguageTxt += tr("Yacc");
  slLanguageTxt += tr("Z Shell");

  cbLanguage->insertStringList(slLanguageTxt);

  // Unprintables
  slUnprintablesTxt += tr("auto");
  slUnprintablesTxt += tr("Unix");
  slUnprintablesTxt += tr("space");
  slUnprintablesTxt += tr("question mark");
  slUnprintablesTxt += tr("octal");
  slUnprintablesTxt += tr("hexadecimal");
  slUnprintablesTxt += tr("Emacs");

  slUnprintablesOpt += tr("");
  slUnprintablesOpt += tr("--non-printable-format=caret");
  slUnprintablesOpt += tr("--non-printable-format=space");
  slUnprintablesOpt += tr("--non-printable-format=question-mark");
  slUnprintablesOpt += tr("--non-printable-format=octal");
  slUnprintablesOpt += tr("--non-printable-format=hexa");
  slUnprintablesOpt += tr("--non-printable-format=emacs");

  cbUnprintables->insertStringList(slUnprintablesTxt);

  // PostScript
  slPostScriptTxt += tr("auto");
  slPostScriptTxt += tr("bold");
  slPostScriptTxt += tr("b/w");
  slPostScriptTxt += tr("colored");
  slPostScriptTxt += tr("diff");
  slPostScriptTxt += tr("gray");
  slPostScriptTxt += tr("gray2");
  slPostScriptTxt += tr("matrix");

  slPostScriptOpt += tr("");
  slPostScriptOpt += tr("--prologue=bold");
  slPostScriptOpt += tr("--prologue=bw");
  slPostScriptOpt += tr("--prologue=color");
  slPostScriptOpt += tr("--prologue=diff");
  slPostScriptOpt += tr("--prologue=gray");
  slPostScriptOpt += tr("--prologue=gray2");
  slPostScriptOpt += tr("--prologue=matrix");

  cbPostScript->insertStringList(slPostScriptTxt);

  // TabSize
  sTabSizeOpt = tr("--tabsize="); 

  // LineNumbers
  sLineNumbersOpt = tr("--line-numbers=");
  
  // TOC
  sTOCOpt = tr("--toc");

  // Truncate
  sTruncateOpt = tr("--truncate-lines=yes");

  // Encoding
  slEncodingTxt += tr("auto");
  slEncodingTxt += tr("ASCII");
  slEncodingTxt += tr("EUC-JP");
  slEncodingTxt += tr("HPRoman");
  slEncodingTxt += tr("IBM-CP437");
  slEncodingTxt += tr("IBM-CP850");
  slEncodingTxt += tr("ISO-8859-1");
  slEncodingTxt += tr("ISO-8859-2");
  slEncodingTxt += tr("ISO-8859-3");
  slEncodingTxt += tr("ISO-8859-4");
  slEncodingTxt += tr("ISO-8859-5");
  slEncodingTxt += tr("ISO-8859-7");
  slEncodingTxt += tr("ISO-8859-9");
  slEncodingTxt += tr("ISO-8859-10");
  slEncodingTxt += tr("ISO-8859-13");
  slEncodingTxt += tr("ISO-8859-15");
  slEncodingTxt += tr("KOI8");
  slEncodingTxt += tr("MS-CP1250");
  slEncodingTxt += tr("Macintosh");

  slEncodingOpt += tr("");
  slEncodingOpt += tr("--encoding=ascii");
  slEncodingOpt += tr("--encoding=euc-jp");
  slEncodingOpt += tr("--encoding=hp");
  slEncodingOpt += tr("--encoding=ibm-cp437");
  slEncodingOpt += tr("--encoding=ibm-cp850");
  slEncodingOpt += tr("--encoding=iso1");
  slEncodingOpt += tr("--encoding=iso2");
  slEncodingOpt += tr("--encoding=iso3");
  slEncodingOpt += tr("--encoding=iso4");
  slEncodingOpt += tr("--encoding=iso5");
  slEncodingOpt += tr("--encoding=iso7");
  slEncodingOpt += tr("--encoding=iso9");
  slEncodingOpt += tr("--encoding=iso10");
  slEncodingOpt += tr("--encoding=iso13");
  slEncodingOpt += tr("--encoding=iso15");
  slEncodingOpt += tr("--encoding=koi8");
  slEncodingOpt += tr("--encoding=ms-cp1250");
  slEncodingOpt += tr("--encoding=mac");

  cbEncoding->insertStringList(slEncodingTxt);

  // Delegate
  sDelegateYesOpt = tr("--delegate=yes");
  sDelegateNoOpt = tr("--delegate=no");
  
  // Prefeed
  sPrefeedOpt = tr("--page-prefeed");

  // Copies
  sCopiesOpt = tr("--copies=");

  // CommandLine
  commandLine();

  // Processes
  procType = new QProcess(this);
  connect(procType, SIGNAL(readyReadStdout()),
	  this, SLOT(readType()) );

  procView = new QProcess(this);
  connect(procView, SIGNAL(processExited()),
	  this, SLOT(readView()) );

  procGen = new QProcess(this);
  connect(procGen, SIGNAL(processExited()),
	  this, SLOT(readGen()) );
  
  sTmpFileName = tmpnam(0);

  // Tipo language
  slTipoLanguageTxt += tr("English");
  slTipoLanguageTxt += tr("Deutsch");
  cbTipoLanguage->insertStringList(slTipoLanguageTxt);
  if(tiInitVals->bFirstStart == true) {
    cbTipoLanguage->setCurrentItem(tiInitVals->iTipoLanguageInit);
  }

  // GUI style
  slGUIStyleTxt = QStyleFactory::keys();
  cbGUIStyle->insertStringList(slGUIStyleTxt);
  if(tiInitVals->bFirstStart == true) {
    cbGUIStyle->setCurrentItem(tiInitVals->iGUIStyleInit);
  }

  // PathA2ps
  if(tiInitVals->bFirstStart == true) {
    lePathA2ps->setText(tiInitVals->sPathA2psInit);
  }

  // PathGv
  if(tiInitVals->bFirstStart == true) {
    lePathGv->setText(tiInitVals->sPathGvInit);
  }

  // First start / normal start.
  if(tiInitVals->bFirstStart) {
    saveTipo(); // generate ~/.tipografrc
  } else {
    loadTipo(); // load ~/.tipografrc
  }

  // Finished start up
  finishedStartUp = true;
}

// ============================================================
// Destructor
// ============================================================
tipografImpl::~tipografImpl()
{
  procType->disconnect();
  delete procType;
  procType = 0;

  procView->disconnect();
  delete procView;
  procView = 0;

  saveTipo();
}
